//+------------------------------------------------------------------+
//|   Last edit: June 29 2014                  Rolling Market.mq4    |
//|   ** Chart Indicator **                                          |
//|   A Rolling Market Meter; MTF, visualizes TF bar construction.   |
//|   Using it with option RollingMarket = true: (by default)        |
//|   makes all options available and shows Rolling Bars.            |
//|   Each Bar always contains all price information over            |
//|   the duration of a full TF! 1 Day=6*H4 candles, H4= 4*H1, etc.  |
//|   This sets up visual ranges proportional in size to TF length.  |
//7                                                                  |
//|   If option RM = false, one will see the Classical Market.       |
//|   Now many Price Bars simply disappear all at the same time      |
//|   on bar changes! Much range/price info is repeatedly lost.      |
//+------------------------------------------------------------------+
#property copyright " 2014 apipshort"
#property link "http://www.forexfactory.com/showthread.php?p=7513155"
#property indicator_chart_window

extern bool   RollingMarket = true; // true
extern string userChoice    = "Number of TFs to Show:";
extern int    maxTF         = 9; // 9
extern int    anchor        = 0; // 0 Moves indicator to the left
extern int    interval      = 4; // 4 Use higher to separate Bars when zoomed out
extern bool   shadowBarOn   = false; // false
extern bool   markLinesOn   = true;  // true
extern bool   lastDayBarOn  = true;  // true
extern bool   showPrice     = true;  // true
extern int    width         = 4; //4 use 2...5
extern string BackGrdColor  = "Dark BG = 1, Light BG = 2";
extern int    colorBG       = 1; // 1
extern int    fontSize      = 8; // 8 use 7...9
extern string font          = "Arial";

int style = 0;

double tfBar[11][3]; // data for Rolling Bars
int formBar[11][2];  // data from Bars of next lower TF

double xPoint;         // convert price values to pips
string pref = "roll_"; // graphical objects prefix
string tF;             // TF names for labels
int per, workP, onBar, last; // PERIOD_M1 etc, number of formBars, index last bar
int iD, thisId;              // code ID for TFs 1 to 9
color clrUp, clrDown, clrYou, clrText, clrMark, clrLastDay; // clrYou = current TF

string padding; // offsetting text from Bar #0 with some left padding
int digs = 0;
//+------------------------------------------------------------------+

void SetColors(int backGround)
   {
   switch (backGround)
      {
      case 1: clrUp=ForestGreen; clrDown=Crimson; clrYou=Gold; clrText=Silver; clrMark=Tomato; clrLastDay=SaddleBrown; return;
      case 2: clrUp=LimeGreen; clrDown=Red; clrYou=Black; clrText=Black; clrMark=Black; clrLastDay=Sienna; return;

      default: return;
      }
   } // end void SetColors(int backGround)

//+------------------------------------------------------------------+

int init()
   {
   //some workaround to show pip values for gold quoted with 2 digits
   //as rounded to dollars and dimes.
   xPoint = Point;
   if(Digits==5||Digits==3) {xPoint = Point * 10;}
   if(Digits==2) {xPoint = Point * 100; digs = 1;}

   if((maxTF <= 0) || (maxTF > 9)) maxTF = 9;

   for(int i=1; i <= maxTF; i++)
      {TF(i);
      if(Period() == per) thisId = iD;
      }

   //if found label "bg_Mark" read BG selection from there
   int getBG = StrToInteger(ObjectDescription("bg_Mark"));
   if(getBG > 0) colorBG = getBG;
   if(colorBG < 1 || colorBG > 2) colorBG = 1;
   SetColors(colorBG);

   padding = "                         "; // adjust as fit
   //........"........................."; place holder only

   return(0);
   } // end init()

//+------------------------------------------------------------------+

int deinit()
   {
   Clear(pref, 0);
   return(0);
   } // end deinit()

//+------------------------------------------------------------------+

int start()
   {
   if(RollingMarket)
      {
      RollingMarket();
         } else {
      ClassicMarket();
      }

   return(0);
   } // end start()

//+------------------------------------------------------------------+

 void ClassicMarket()
   { // barebones, real time TF Bars only

   TF(thisId);
   interval = anchor;

   for(int i=1; i <= maxTF; i++)
      {
      TF(i);

      double hi   = iHigh(NULL, per, 0);
      double low   = iLow(NULL, per, 0);
      double open = iOpen(NULL, per, 0);

      color clrX = SeaGreen ;

      string obj = StringConcatenate("Open_", tF);
      TrendLine(obj, interval + 1, interval, open, open, clrX, style, width - 1, false, false);

      if(Period() == per) clrX = clrYou;
      TrendLine(tF, interval, interval, hi, low, clrX, style, width, false, false);

      obj = StringConcatenate("t_", tF);
      TextBox(obj, interval, low, tF, fontSize, font, clrText);

      obj = StringConcatenate("p_", tF);
      TextBox(obj, interval + 1, hi, DoubleToStr((hi - low) / xPoint, digs), fontSize, font, clrText);

      interval += 4;
      } // end for(int i=1; i <= maxTF; i++)

   return;
   } // end void ClassicMarket()

//+------------------------------------------------------------------+

void RollingMarket()
   { // has complete functionality & shows Rolling Bars
     // on new tick load arrays with data for TFs

   TF(thisId);

   TF(1); // get real data for M1, not a rolling bar!
   tfBar[1][0] = iHigh(NULL,per,0);
   tfBar[1][1] =  iLow(NULL,per,0);
   tfBar[1][2] = iOpen(NULL,per,0);

   for(int i=2; i <= maxTF; i++)
      { // get all other data & index of formBars in lower TF
      TF(i);
      int indexHi = iHighest(NULL, workP, MODE_HIGH, onBar, 0);
      tfBar[i][0] = iHigh(NULL, workP, indexHi);

      int indexLo = iLowest(NULL, workP, MODE_LOW, onBar, 0);
      tfBar[i][1] = iLow(NULL, workP, indexLo);

      tfBar[i][2] = iOpen(NULL, workP, onBar-1);

      formBar[i][0] = indexHi;
      formBar[i][1] = indexLo;
      } // end for(int i=2; i <= maxTF; i++)

   ManageBars();

   return;
   } // end void RollingMarket()

//+------------------------------------------------------------------+

 void ManageBars()
    { // using current tick data to update only changed objects.

    for(int i=1; i <= maxTF; i++)
      {
      TF(i);
      int pBar = anchor + ((iD - 1) * interval); // set chart bar on which to display rolling bar

      if(Bid > tfBar[i][2])  color clr = clrUp;
      if(Bid <= tfBar[i][2]) clr = clrDown;

      // bar Open markers
      string obj = StringConcatenate("Open_", tF);
      TrendLine(obj, pBar + 1, pBar, tfBar[i][2], tfBar[i][2], clr, style, width - 1, false, false);

      // rolling bars
      if(Period() == per) clr = clrYou; // color of current TF
      TrendLine(tF, pBar, pBar, tfBar[i][0], tfBar[i][1], clr, style, width, false, false);

      // labels for bar names and pip values
      obj = StringConcatenate("p_", tF); // pips on bar
      string text = DoubleToStr((tfBar[i][0] - tfBar[i][1]) / xPoint, digs);
      TextBox(obj, pBar+1, tfBar[i][0], text, fontSize, font, clrText);

      obj = StringConcatenate("t_", tF); // TF names
      TextBox(obj, pBar, tfBar[i][1], tF, fontSize, font, clrText);

      // More
      if(showPrice)
         { // show rounded price on the right
         text = StringConcatenate(padding, DoubleToStr(Bid, Digits - 1));
         TextBox("price", 0, Bid, text, fontSize + 2, font, clrText);
         }// end if(showPrice)

      if(shadowBarOn && Period() == per)
         { // shadow bar
         TrendLine("ShadowBar", 0, 0, tfBar[i][0], tfBar[i][1], clrYou, style, 1, false, false);
         } // end if(shadowBarOn

      if(markLinesOn && Period() == per && thisId < maxTF)
         { // show Mark Lines from next higher tf to formBars
         TrendLine("Hi", pBar + interval, formBar[i+1][0], tfBar[i+1][0], tfBar[i+1][0], clrMark, 2, 0, false, true);
         TrendLine("Lo", pBar + interval, formBar[i+1][1], tfBar[i+1][1], tfBar[i+1][1], clrMark, 2, 0, false, true);

         Arrow("lastBar", 115, last, iLow(NULL,per,last), clrYou, width - 1);// 217
         } // end if(markLinesOn

      if(lastDayBarOn && iD == 7)
         { // real Last Day Bar with Open and Close
         TrendLine("LastDay",       pBar + 2, pBar + 2,  iHigh(NULL,per,1),   iLow(NULL,per,1), clrLastDay, style, width - 1, false, false);
         TrendLine("Open_Lastday",  pBar + 3, pBar + 2,  iOpen(NULL,per,1),  iOpen(NULL,per,1), clrLastDay, style, width - 1, false, false);
         TrendLine("Close_Lastday", pBar + 2, pBar + 1, iClose(NULL,per,1), iClose(NULL,per,1), clrLastDay, style, width - 1, false, false);

         // Last Day Bar's pip size
         text = DoubleToStr((iHigh(NULL,per,1) - iLow(NULL,per,1)) / xPoint, digs);
         TextBox("p_LastDay", pBar + 3, iHigh(NULL,per,1), text, fontSize, font, clrText);

         // pips currently away from Daily Open
         text = DoubleToStr((Bid - iOpen(NULL,per,0)) / xPoint, digs);
         TextBox("p_pips_Daily_Open", pBar + 3, iClose(NULL,per,1), text, fontSize, font, clrText);
         } // end if(lastDayBarOn

      } // end for(int i=1; i <= maxTF; i++)

   return;
   } // end void ManageBars()

//+---------------------------FUNCTIONS------------------------------+

void TrendLine(string obj, int t1, int t2, double p1, double p2, color clr, int st, int w, bool ray, bool back)
   {
   obj = StringConcatenate(pref, obj);
   if(!ObjectCreate(obj, OBJ_TREND,0,0,0,0,0))  ObjectCreate(obj, OBJ_TREND,0,0,0,0,0);
      if(!ObjectSet(obj, OBJPROP_TIME1, Time[t1])) ObjectSet(obj, OBJPROP_TIME1, Time[t1]);
      if(!ObjectSet(obj, OBJPROP_TIME2, Time[t2])) ObjectSet(obj, OBJPROP_TIME2, Time[t2]);
      if(!ObjectSet(obj, OBJPROP_PRICE1, p1))      ObjectSet(obj, OBJPROP_PRICE1, p1);
      if(!ObjectSet(obj, OBJPROP_PRICE2, p2))      ObjectSet(obj, OBJPROP_PRICE2, p2);
      if(!ObjectSet(obj, OBJPROP_COLOR,  clr))     ObjectSet(obj, OBJPROP_COLOR,  clr);
      if(!ObjectSet(obj, OBJPROP_STYLE,  st))      ObjectSet(obj, OBJPROP_STYLE,  st);
      if(!ObjectSet(obj, OBJPROP_WIDTH,  w))       ObjectSet(obj, OBJPROP_WIDTH,  w);
      if(!ObjectSet(obj, OBJPROP_RAY,    ray))     ObjectSet(obj, OBJPROP_RAY,    ray);
      if(!ObjectSet(obj, OBJPROP_BACK,   back))    ObjectSet(obj, OBJPROP_BACK,   back);

   return;
   }// end void TrendLine(

//+------------------------------------------------------------------+

void TextBox(string obj, int t1, double p1, string text, int size, string afont, color clr)
   {
   obj = StringConcatenate(pref, obj);
   if (!ObjectCreate(obj, OBJ_TEXT, 0,0,0,0))    ObjectCreate(obj, OBJ_TEXT, 0,0,0,0);
       if(!ObjectSet(obj, OBJPROP_TIME1, Time[t1])) ObjectSet(obj, OBJPROP_TIME1, Time[t1]);
       if(!ObjectSet(obj, OBJPROP_PRICE1, p1))      ObjectSet(obj, OBJPROP_PRICE1, p1);
   if(!ObjectSetText(obj, text, size, afont, clr))  ObjectSetText(obj, text, size, afont, clr);

   return;
   }// end void TextBox(

//+------------------------------------------------------------------+

void Arrow(string obj, int code, int t1, double p1, color clr, int w)
   {
   obj = StringConcatenate(pref, obj);
      if(!ObjectCreate(obj, OBJ_ARROW, 0,0,0))     ObjectCreate(obj, OBJ_ARROW, 0,0,0);
         if(!ObjectSet(obj, OBJPROP_ARROWCODE, code)) ObjectSet(obj, OBJPROP_ARROWCODE, code);
         if(!ObjectSet(obj, OBJPROP_TIME1, Time[t1])) ObjectSet(obj, OBJPROP_TIME1, Time[t1]);
         if(!ObjectSet(obj, OBJPROP_PRICE1, p1))      ObjectSet(obj, OBJPROP_PRICE1, p1);
         if(!ObjectSet(obj, OBJPROP_COLOR, clr))      ObjectSet(obj, OBJPROP_COLOR, clr);
         if(!ObjectSet(obj, OBJPROP_WIDTH, w))        ObjectSet(obj, OBJPROP_WIDTH, w);

   return;
   }// end void Arrow(

//+---------------------SUPPORT--------------------------------------+

void TF(int timeFrame)
   {// DO NOT MESS
   switch (timeFrame)
      {
      case 1: iD=1; tF="M1";  per=PERIOD_M1;  workP=PERIOD_M1;  onBar=0; last=4; return;
      case 2: iD=2; tF="M5";  per=PERIOD_M5;  workP=PERIOD_M1;  onBar=5; last=2; return;
      case 3: iD=3; tF="M15"; per=PERIOD_M15; workP=PERIOD_M5;  onBar=3; last=1; return;
      case 4: iD=4; tF="M30"; per=PERIOD_M30; workP=PERIOD_M15; onBar=2; last=1; return;
      case 5: iD=5; tF="H1";  per=PERIOD_H1;  workP=PERIOD_M30; onBar=2; last=3; return;
      case 6: iD=6; tF="H4";  per=PERIOD_H4;  workP=PERIOD_H1;  onBar=4; last=5; return;
      case 7: iD=7; tF="D1";  per=PERIOD_D1;  workP=PERIOD_H4;  onBar=6; last=4; return;
      case 8: iD=8; tF="W1";  per=PERIOD_W1;  workP=PERIOD_D1;  onBar=5; last=3; return;
      case 9: iD=9; tF="MN";  per=PERIOD_MN1; workP=PERIOD_W1;  onBar=4; last=0; return;

      default: return;
      }
   } // end void TF(int timeFrame)

//+------------------------------------------------------------------+

void Clear(string findMe, int startFrom)
   {
   string name;
   int obj_total = ObjectsTotal();
   for (int i=obj_total-1; i>=0; i--)
      {
       name = ObjectName(i);
       if (StringFind(name, findMe) == startFrom) ObjectDelete(name);
      }
   return;
   } // end void Clear(string findMe, int startFrom)

//+-----------------------------EOP----------------------------------+
